﻿using Maynot.WPF.ViewModel;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Maynot.WPF.View
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += MainWindow_Loaded;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            var storyboard = (Storyboard)FindResource("EnlargeAndShrinkAnimation");
            storyboard.Begin(play, true);
            play.Click += Play_Click;
        }
        private void Play_Click(object sender, RoutedEventArgs e)
        {
            var storyboard = (Storyboard)FindResource("EnlargeAndShrinkAnimation");
            storyboard.Stop(play);
            play.Click -= Play_Click;  // Remove the click event handler so that the animation is not stopped again
        }

        private void ScrollViewer_PreviewMouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (e.Delta > 0)
            {
                // Zoom in
                Viewbox.Width = Math.Min(Viewbox.ActualWidth * 1.1, Viewbox.ActualWidth * 5);
                Viewbox.Height = Math.Min(Viewbox.ActualHeight * 1.1, Viewbox.ActualHeight * 5);
            }
            else
            {
                // Zoom out
                Viewbox.Width = Math.Max(Viewbox.ActualWidth / 1.1, Viewbox.ActualWidth / 5);
                Viewbox.Height = Math.Max(Viewbox.ActualHeight / 1.1, Viewbox.ActualHeight / 5);
            }

            e.Handled = true;
        }

        private bool isDragging = false;
        private Point startPoint;

        private void Popup_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Left)
            {
                isDragging = true;
                startPoint = e.GetPosition(TransactionsPopup);
            }
        }

        private void Popup_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                var currentPoint = e.GetPosition(TransactionsPopup);
                var offset = currentPoint - startPoint;

                TransactionsPopup.HorizontalOffset += offset.X;
                TransactionsPopup.VerticalOffset += offset.Y;

                startPoint = currentPoint;
            }
        }

        private void Popup_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            isDragging = false;
        }

        private Point scrollMousePoint = new Point();
        private double hOff = 1;
        private double vOff = 1;

        private void ScrollViewer_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
            {
                scrollMousePoint = e.GetPosition((ScrollViewer)sender);
                hOff = ((ScrollViewer)sender).HorizontalOffset;
                vOff = ((ScrollViewer)sender).VerticalOffset;
                ((ScrollViewer)sender).CaptureMouse();
            }
        }

        private void ScrollViewer_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            if (((ScrollViewer)sender).IsMouseCaptured &&
                (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl)))
            {
                ((ScrollViewer)sender).ScrollToHorizontalOffset(hOff + (scrollMousePoint.X - e.GetPosition((ScrollViewer)sender).X));
                ((ScrollViewer)sender).ScrollToVerticalOffset(vOff + (scrollMousePoint.Y - e.GetPosition((ScrollViewer)sender).Y));
            }
        }

        private void ScrollViewer_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            ((ScrollViewer)sender).ReleaseMouseCapture();
        }


    }

}
