﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data.Common;
using System.Diagnostics;
using System.Linq;
using System.Reflection.Metadata;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Maynot.WPF.ViewModel.MaynotTileClasses;
using Maynot.WPF.ViewModel.MaynotTileClasses.Facilities;
using Maynot.WPF.ViewModel.MaynotTileClasses.Zones;
using MaynotModel;
using static Maynot.WPF.ViewModel.MaynotTileClasses.Zones.Zone;
using Point = System.Drawing.Point;

namespace Maynot.WPF.ViewModel
{
    class MaynotViewModel : ViewModelBase
    {
        private MaynotGameModel _model;
        private bool currentlyAnimating = false;
        /// <summary>
        /// Property for fileds.
        /// </summary>
        public ObservableCollection<MaynotTile> Fields { get; set; }
        /// <summary>
        /// Property for people in <see cref="Zone"/>.
        /// </summary>
        public ObservableCollection<Person> SatisfactionsInInspectedZone { get; set; }
        /// <summary>
        /// Property for transactions.
        /// </summary>
        public ObservableCollection<Transaction> Transactions { get; set; }
        /// <summary>
        /// Property for capacity of <see cref="Zone"/>.
        /// </summary>
        public String InspectedZoneCapacity { get; set; }
        /// <summary>
        /// Property for fullness of <see cref="Zone"/>.
        /// </summary>
        public String InspectedZoneFullness { get; set; }
        /// <summary>
        /// Property for level of <see cref="Zone"/>.
        /// </summary>
        public String InspectedZoneLevel { get; set; }
        /// <summary>
        /// Property for upgrading cost of <see cref="Zone"/>.
        /// </summary>
        public String InspectedUpgradePrice { get; set; }
        /// <summary>
        /// Property getter of budeget in <see cref="_model"/>.
        /// </summary>
        public float Money { get { return _model.Money; } }
        /// <summary>
        /// Property getter of <see cref="ResidentialZone"/> taxrate in <see cref="_model"/>.
        /// </summary>
        public float ResidentalTax { get { return _model.ResidentalTax; } }
        /// <summary>
        /// Property getter of <see cref="ServiceZone"/> taxrate in <see cref="_model"/>.
        /// </summary>
        public float ServiceTax { get { return _model.ServiceTax; } }
        /// <summary>
        /// Property getter of <see cref="IndustrialZone"/> taxrate in <see cref="_model"/>.
        /// </summary>
        public float IndustrialTax { get { return _model.IndustrialTax; } }
        /// <summary>
        /// Property getter of in-game time.
        /// </summary>
        public String Date { get; set; }
        /// <summary>
        /// Property getter of game speed.
        /// </summary>
        public float Speed { get { return _model.Speed; } }
        /// <summary>
        /// Property getter of number of citizens.
        /// </summary>
        public int Population { get { return _model.Population; } }
        /// <summary>
        /// Property getter of average satisfaction of citizens.
        /// </summary>
        public double Satisfaction { get { return _model.AverageSatisfaction; } }
        private Point currentlyInspectedZone = new Point(0, 0);

        private MaynotTile? _selectedTile;

        /// <summary>
        /// Property of <see cref="_selectedTile"/>.
        /// </summary>
        public MaynotTile? SelectedTile
        {
            get { return _selectedTile; }
            set
            {
                _selectedTile = value;
                OnPropertyChanged(nameof(SelectedTile));
                UpdateBalPanelBackGroundColors();
            }
        }

        private SolidColorBrush _balPanelAltalanosGombBackground = MaynotTile.BalPanelHighlightColor;
        /// <summary>
        /// Property of <see cref="_balPanelAltalanosGombBackground"/>.
        /// </summary>
        public SolidColorBrush BalPanelAltalanosGombBackground
        {
            get { return _balPanelAltalanosGombBackground; }
            set
            {
                _balPanelAltalanosGombBackground = value;
                OnPropertyChanged();
            }
        }
        private SolidColorBrush _balPanelBulldozerGombBackground = MaynotTile.BalPanelDefaultColor;
        /// <summary>
        /// Property of <see cref="_balPanelBulldozerGombBackground"/>.
        /// </summary>
        public SolidColorBrush BalPanelBulldozerGombBackground
        {
            get { return _balPanelBulldozerGombBackground; }
            set
            {
                _balPanelBulldozerGombBackground = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// Radio button command property.
        /// </summary>
        public ICommand RadioButtonCheckedCommand { get; set; }
        /// <summary>
        /// Placing item command property.
        /// </summary>
        public ICommand PlaceItemCommand { get; set; }

        private ObservableCollection<MaynotTile> _letehetoElemek;

        /// <summary>
        /// Placable items on building panel.
        /// </summary>
        public ObservableCollection<MaynotTile> LetehetoElemek
        {
            get { return _letehetoElemek; }
            set
            {
                _letehetoElemek = value;
                OnPropertyChanged(nameof(LetehetoElemek));
            }
        }

        private bool _isHelpPopupOpen;
        /// <summary>
        /// Property of <see cref="_isHelpPopupOpen"/>.
        /// </summary>
        public bool IsHelpPopupOpen
        {
            get { return _isHelpPopupOpen; }
            set
            {
                _isHelpPopupOpen = value;
                OnPropertyChanged(); // Notify that property has changed
            }
        }
        private bool _isTransactionsPopupOpen;
        /// <summary>
        /// Property of <see cref="_isTransactionsPopupOpen"/>.
        /// </summary>
        public bool IsTransactionsPopupOpen
        {
            get { return _isTransactionsPopupOpen; }
            set
            {
                _isTransactionsPopupOpen = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// Property of first time playing.
        /// </summary>
        public bool IsFirstTimePlaying
        {
            get { return Properties.Settings.Default.IsFirstTimePlaying; }
            set
            {
                Properties.Settings.Default.IsFirstTimePlaying = value;
                Properties.Settings.Default.Save();
            }
        }

        /// <summary>
        /// Close helping popup.
        /// </summary>
        public DelegateCommand CloseHelpPopupCommand { get; private set; }
        /// <summary>
        /// Cunstructor.
        /// </summary>
        /// <param name="model">takes in and initializes a <see cref="MaynotModel"/></param>
        public MaynotViewModel(MaynotGameModel model)
        {
            _model = model;
            _model.DayElapsed += new EventHandler<TimeElapsedEventArgs>(Day_Elapsed);
            _model.GameSpeedChanged += new EventHandler<EventArgs>(Speed_Changed);
            _model.UpdatePopulation += new EventHandler<EventArgs>(Population_Changed);
            _model.InspectedTileChanged += new EventHandler<InspectedTileChangedEventArgs>(InspectedTile_Changed);
            _model.TransactionHappenned += new EventHandler<MaynotModel.TransactionEventArgs>(Transactions_Changed);
            // parancsok kezelése
            NewGameCommand = new DelegateCommand(param => OnNewGame());
            LoadGameCommand = new DelegateCommand(param => OnLoadGame());
            SaveGameCommand = new DelegateCommand(param => OnSaveGame());
            ExitGameCommand = new DelegateCommand(param => OnExitGame());
            SlowerGameCommand = new DelegateCommand(param => OnSlowerGame());
            SpeedUpGameCommand = new DelegateCommand(param => OnSpeedUpGameCommand());
            PauseGameCommand = new DelegateCommand(param => OnPauseGameCommand());
            ResumeGameCommand = new DelegateCommand(param => OnResumeGame());
            ResidentalTaxNumericUpCommand = new DelegateCommand(param => OnResidentalTaxNumericUp());
            ResidentalTaxNumericDownCommand = new DelegateCommand(param => OnResidentalTaxNumericDown());
            ServiceTaxNumericUpCommand = new DelegateCommand(param => OnServiceTaxNumericUp());
            ServiceTaxNumericDownCommand = new DelegateCommand(param => OnServiceTaxNumericDown());
            IndustrialTaxNumericUpCommand = new DelegateCommand(param => OnIndustrialTaxNumericUp());
            IndustrialTaxNumericDownCommand = new DelegateCommand(param => OnIndustrialTaxNumericDown());
            EscapeKeyPressedCommand = new DelegateCommand(param => OnEscapeKeyPressed());
            OpenHelpPopupCommand = new DelegateCommand(param => OnOpenHelpPopupCommand());
            CloseHelpPopupCommand = new DelegateCommand(param => OnCloseHelpPopupCommand());
            SelectBulldozerCommand = new DelegateCommand(param => OnSelectBulldozerCommand());
            UpgradeInspectedZoneCommand = new DelegateCommand(param => OnUpgradeInspectedZone());
            OpenTransactionsPopupCommand = new DelegateCommand(param => OnOpenTransactionsPopupCommand());
            CloseTransactionsPopupCommand = new DelegateCommand(param => OnCloseTransactionsPopupCommand());


            Fields = new ObservableCollection<MaynotTile>();
            SatisfactionsInInspectedZone = new ObservableCollection<Person>();
            Transactions = new ObservableCollection<Transaction>();
            FullyRefreshTable();

            RadioButtonCheckedCommand = new DelegateCommand(OnRadioButtonChecked);
            PlaceItemCommand = new DelegateCommand(OnPlaceItem);

            LetehetoElemek = new ObservableCollection<MaynotTile>
            {
                new Road(),
                new ResidentialZone(),
                new IndustrialZone(),
                new ServiceZone(),
                new PoliceStation(),
                new Stadium(),
                new School(),
                new University(),
                new Forest()
            };
            //DebugTriggerCatastropheHappened(6,6);
        }

        private void Speed_Changed(object sender, EventArgs e)
        {
            OnPropertyChanged(nameof(Speed));
        }
        private void InspectedTile_Changed(object sender, InspectedTileChangedEventArgs e)
        {
            App.Current?.Dispatcher.Invoke((Action)delegate
            {
                SatisfactionsInInspectedZone.Clear();
            });
            if (e._tile is MaynotPersistence.Zone zone)
            {
                InspectedZoneCapacity = zone.Capacity.ToString();
                InspectedZoneFullness = zone.GetPeoples(_model.Citizens).Count().ToString();
                InspectedZoneLevel = ((int)zone.Level).ToString();
                InspectedUpgradePrice = zone.UpgradeCost.ToString();
                ObservableCollection<Person> tmpPeople = new ObservableCollection<Person>(); ;
                foreach (var person in zone.GetPeoples(_model.Citizens))
                {
                    tmpPeople.Add(new Person(person));
                }
                App.Current?.Dispatcher.Invoke((Action)delegate
                {
                    SatisfactionsInInspectedZone = new ObservableCollection<Person>(tmpPeople);
                });
            }
            else
            {
                InspectedZoneCapacity = InspectedZoneFullness = InspectedZoneLevel = InspectedUpgradePrice = String.Empty;
                App.Current?.Dispatcher.Invoke((Action)delegate
                {
                    SatisfactionsInInspectedZone.Clear();
                });
            }
            OnPropertyChanged(nameof(InspectedZoneCapacity));
            OnPropertyChanged(nameof(InspectedZoneFullness));
            OnPropertyChanged(nameof(InspectedZoneLevel));
            OnPropertyChanged(nameof(InspectedUpgradePrice));
            OnPropertyChanged(nameof(SatisfactionsInInspectedZone));
            if (IsFirstTimePlaying) //Amennyiben most nyitják meg előszőr az applikációt
            {
                OnOpenHelpPopupCommand(); //Megmutatjuk a help popupot
                IsFirstTimePlaying = false;
            }
        }
        private void UpdateHighlightsOnGrid(Point torlendo, Point ujHighlightolando)
        {
            SetVisibilityAtCoordinates(torlendo.X, torlendo.Y, Visibility.Hidden);
            SetVisibilityAtCoordinates(ujHighlightolando.X, ujHighlightolando.Y, Visibility.Visible);
        }
        private void SetCurrentInspectedHighlightOnGrid()
        {
            if (currentlyInspectedZone != new Point(0, 0))
            {
                SetVisibilityAtCoordinates(currentlyInspectedZone.X, currentlyInspectedZone.Y, Visibility.Visible);
            }            
        }
        private void Population_Changed(object sender, EventArgs e)
        {
            OnPropertyChanged(nameof(Population));
            UpdateTable();
        }
        private void Day_Elapsed(object sender, TimeElapsedEventArgs e)
        {
            Date = e.newDate.ToString("yyyy MMMM dd");
            OnPropertyChanged(nameof(Date));
            OnPropertyChanged(nameof(Money));
            OnPropertyChanged(nameof(Satisfaction));
        }
        private void OnResidentalTaxNumericUp()
        {
            _model.setResidentalTax(_model.ResidentalTax + 1);
            OnPropertyChanged(nameof(ResidentalTax));
        }
        private void OnUpgradeInspectedZone()
        {
            _model.UpgradeInspectedZone();
        }
        private void OnResidentalTaxNumericDown()
        {
            _model.setResidentalTax(_model.ResidentalTax - 1);
            OnPropertyChanged(nameof(ResidentalTax));
        }
        private void OnIndustrialTaxNumericUp()
        {
            _model.setIndustrialTax(_model.IndustrialTax + 1);
            OnPropertyChanged(nameof(IndustrialTax));
        }
        private void OnIndustrialTaxNumericDown()
        {
            _model.setIndustrialTax(_model.IndustrialTax - 1);
            OnPropertyChanged(nameof(IndustrialTax));
        }
        private void OnServiceTaxNumericUp()
        {
            _model.setServiceTax(_model.ServiceTax + 1);
            OnPropertyChanged(nameof(ServiceTax));
        }
        private void OnServiceTaxNumericDown()
        {
            _model.setServiceTax(_model.ServiceTax - 1);
            OnPropertyChanged(nameof(ServiceTax));
        }
        private void OnResumeGame()
        {
            _model.resumeTime();
        }

        private void OnPauseGameCommand()
        {
            _model.stopTime();
        }

        private void OnSpeedUpGameCommand()
        {
            _model.speedUpTime();
        }

        private void OnSlowerGame()
        {
            _model.slowTime();
        }

        private void OnRadioButtonChecked(object? selectedItem)
        {
            if (selectedItem is MaynotTile maynotTile)
            {
                Debug.WriteLine($"Klikkelve: {maynotTile.Name}");
                SelectedTile = maynotTile;
            }
            else
            {
                throw new ArgumentException("Olyan elemen jött be 'OnRadioButtonChecked' esemény amely nem MaynotTile típusú!");
            }
        }

        private void UpdateBalPanelBackGroundColors()
        {
            MaynotTile? tileToBeHighlighted = SelectedTile;
            foreach (MaynotTile? item in LetehetoElemek)
            {
                if (item != tileToBeHighlighted)
                {
                    item.BalPanelBackground = MaynotTile.BalPanelDefaultColor; 
                }
                else
                {
                    item.BalPanelBackground = MaynotTile.BalPanelHighlightColor;
                }
            }
            BalPanelBulldozerGombBackground = MaynotTile.BalPanelDefaultColor;
            BalPanelAltalanosGombBackground = MaynotTile.BalPanelDefaultColor;
            if (tileToBeHighlighted is Bulldozer)
            {
                BalPanelBulldozerGombBackground = MaynotTile.BalPanelHighlightColor;
            }
            if (tileToBeHighlighted is null)
            {
                BalPanelAltalanosGombBackground = MaynotTile.BalPanelHighlightColor;
            }
        }

        private void OnEscapeKeyPressed()
        {
            if (IsHelpPopupOpen)
            {
                OnCloseHelpPopupCommand();
            }
            else
            {
                ClearCurrentlySelectedTile();
            }
        }

        private void ClearCurrentlySelectedTile()
        {
            SelectedTile = null;
        }

        private void OnOpenHelpPopupCommand()
        {
            IsHelpPopupOpen = true;
        }

        private void OnCloseHelpPopupCommand()
        {
            IsHelpPopupOpen = false;
        }
        private void OnOpenTransactionsPopupCommand()
        {
            IsTransactionsPopupOpen = true;
            _model.Transactions.ForEach((transaction) =>
            {
                Transactions.Add(new Transaction(transaction.Type, transaction.TransactionAmount, transaction.TransactionDate, transaction.TileType));
            });
        }
        private void Transactions_Changed(object sender, MaynotModel.TransactionEventArgs e)
        {
            if (IsTransactionsPopupOpen)
            {
                App.Current?.Dispatcher.Invoke((Action)delegate
                {
                    Transactions.Add(new Transaction(e.transaction.Type, e.transaction.TransactionAmount, e.transaction.TransactionDate, e.transaction.TileType));
                });
                OnPropertyChanged(nameof(Transactions));
            }
        }
        private void OnCloseTransactionsPopupCommand()
        {
            IsTransactionsPopupOpen = false;
            Transactions.Clear();
        }

        private void OnSelectBulldozerCommand()
        {
            SelectedTile = new Bulldozer(); 
        }

        private void OnPlaceItem(object? selectedItem)
        {
            // Handle placing the selected item on the grid
        }
        /// <summary>
        /// Refreshes all fields in <see cref="Fields"/>.
        /// </summary>
        public void FullyRefreshTable()
        {
            App.Current?.Dispatcher.Invoke((Action)delegate
            {
                Fields.Clear();
                for (Int32 i = 0; i < _model.GameBoard.GetLength(0); i++) // inicializáljuk a mezőket
                {
                    for (Int32 j = 0; j < _model.GameBoard.GetLength(0); j++)
                    {
                        Fields.Add(new Empty
                        {
                            X = i,
                            Y = j,
                            ClickCommand = new DelegateCommand((param)=> PlaceTile(param as MaynotTile))
                        });
                    }
                }
               
            });
            OnPropertyChanged(nameof(Money));
            UpdateTable();
        }

        private void UpdateTable()
        {
            Application.Current.Dispatcher.Invoke(() => UpdateTableLocal());
        }

        private void UpdateTableLocal()
        {
            if (currentlyAnimating) // Éppen animálja a robbanást (katasztrofa)
            {
                return;
            }
            int numOfRoads = 0;
            for (int i = 0; i < Fields.Count; i++) // Előszőr friss állapotra hozzuk a mi Fields-ünket a Model tábla alapján
            {
                MaynotTile tile = Fields[i];
                MaynotTile modelbolTile = GetTileAtCoordinatesFromModel(tile.X, tile.Y);
                Fields[i] = modelbolTile;

            }
            SetCurrentInspectedHighlightOnGrid();
            foreach (MaynotTile tile in Fields) // Aztán frissítjük a Road sprite-okat
            {
                if (tile is Road road)
                {
                    //Debug.WriteLine("A Fieldsben talalt ut koodinatai: " + road.X + " " + road.Y);
                    bool north = false, east = false, south = false, west = false;
                    int fieldSize = _model.GameBoardSize;

                    if (road.X == _model.GetStartRoadX() && road.Y == _model.GetStartRoadY()) //Amennyiben pont a kezdő utat nézzük, akkor annak berakunk egy képzeletbeli west-et hogy jól nézzen ki a tiling
                    {
                        west = true;
                    }

                    if (road.Y > 0 && GetFieldAtCoordinates(road.X, road.Y - 1) is not Empty)
                    {
                        west = true;
                    }
                    if (road.Y < (fieldSize - 1) && GetFieldAtCoordinates(road.X, road.Y + 1) is not Empty)
                    {
                        east = true;
                    }
                    if (0 < road.X && GetFieldAtCoordinates(road.X - 1, road.Y) is not Empty)
                    {
                        north = true;
                    }
                    if (road.X < (fieldSize - 1) && GetFieldAtCoordinates(road.X + 1, road.Y) is not Empty)
                    {
                        south = true;
                    }
                    road.SetRoadSprite(north, east, south, west);
                    numOfRoads++;
                }
            }
            //Debug.WriteLine("Number of Roads: " + numOfRoads);
            OnPropertyChanged(nameof(Money));
            OnPropertyChanged(nameof(Fields));
        }

        private void PlaceTile(MaynotTile tile)
        {
            if (currentlyAnimating) // Éppen a katsztrófát animálja, ekkor nem szabad place-elni
            {
                return;
            }
            //Debug.WriteLine("Clicked on: " + tile.X + " " + tile.Y);
            //Debug.WriteLine("Selected Tile is: " + SelectedTile.GetType());
            if (SelectedTile is Road)
            {
                _model.placeRoad(tile.X, tile.Y);
            }
            else if (SelectedTile is Zone zone)
            {
                if (zone is ResidentialZone)
                {
                    _model.placeResidentialZone(tile.X, tile.Y);
                }
                else if (zone is IndustrialZone)
                {
                    _model.placeIndustrialZone(tile.X, tile.Y);
                }
                else if (zone is ServiceZone)
                {
                    _model.placeServiceZone(tile.X, tile.Y);
                }
            }
            else if (SelectedTile is Facility facility)
            {
                if (facility is PoliceStation)
                {
                    _model.placePoliceStation(tile.X, tile.Y);
                }
                else if (facility is Stadium)
                {
                    _model.placeStadium(tile.X, tile.Y);
                }
                else if (facility is School)
                {
                    _model.placeSchool(tile.X, tile.Y);
                }
                else if (facility is University)
                {
                    _model.placeUni(tile.X, tile.Y);
                }
            }
            else if (SelectedTile is Forest)
            {
                _model.placeForest(tile.X, tile.Y);
            }
            else if (SelectedTile is Bulldozer)
            {
                _model.DestroyTile(tile.X, tile.Y, false);
            }
            else if (SelectedTile is null)
            {
                UpdateHighlightsOnGrid(currentlyInspectedZone, new Point(tile.X, tile.Y));
                _model.InspectZone(tile.X, tile.Y);
                currentlyInspectedZone = new Point(tile.X, tile.Y);
            }
            MaynotTile modelbolTile = GetTileAtCoordinatesFromModel(tile.X, tile.Y);
            //int fieldMeret = (int) (Math.Sqrt(Fields.Count));
            //Fields[tile.X * fieldMeret + tile.Y] = modelbolTile; // Itt nagyon gagyin felülírjuk a Fields listában lévő MaynotTile-t a mi tile-unkra
            //OnPropertyChanged(nameof(Fields));
            UpdateTable();

        }

        private MaynotTile ModelTileToMaynotTile(MaynotPersistence.Tile tile) //Itt nem fogjuk az újonnan létrehozott instance-nek átadni a pozíciót a táblán
        {
            if (tile is MaynotPersistence.Empty) return new Empty();
            if (tile is MaynotPersistence.Road) return new Road();
            if (tile is MaynotPersistence.ResidentialZone residental) return new ResidentialZone(residental.GetPeoples(_model.Citizens).Count, ModelZoneLevelToView(residental.Level));
            if (tile is MaynotPersistence.IndustrialZone industrial) return new IndustrialZone(industrial.GetPeoples(_model.Citizens).Count, ModelZoneLevelToView(industrial.Level));
            if (tile is MaynotPersistence.ServiceZone service) return new ServiceZone(service.GetPeoples(_model.Citizens).Count, ModelZoneLevelToView(service.Level));
            if (tile is MaynotPersistence.Forest) return new Forest(10, 10);
            if (tile is MaynotPersistence.PoliceStation) return new PoliceStation();
            if (tile is MaynotPersistence.Stadium) return new Stadium();
            if (tile is MaynotPersistence.School) return new School();
            if (tile is MaynotPersistence.University) return new University();
            
            return new Road(30);
        }

        private static ZoneLevelsView ModelZoneLevelToView(MaynotPersistence.ZoneLevel zoneLevel)
        {
            switch (zoneLevel)
            {
                case MaynotPersistence.ZoneLevel.SMALL:
                    return ZoneLevelsView.SMALL;
                case MaynotPersistence.ZoneLevel.MEDIUM:
                    return ZoneLevelsView.MEDIUM;
                case MaynotPersistence.ZoneLevel.LARGE:
                    return ZoneLevelsView.LARGE;
                default:
                    return ZoneLevelsView.SMALL;
                    break;
            }
        }

        private MaynotTile GetTileAtCoordinatesFromModel(int x, int y)
        {
            MaynotTile tile = ModelTileToMaynotTile(_model.GameBoard[x, y]);
            tile.X = x;
            tile.Y = y;
            tile.ClickCommand = new DelegateCommand((param) => PlaceTile(param as MaynotTile));
            return tile;
        }
        /// <summary>
        /// Get field from <see cref="Fields"/>.
        /// </summary>
        /// <param name="x">row count</param>
        /// <param name="y">column count</param>
        public MaynotTile GetFieldAtCoordinates(int x, int y)
        {
            int sorHossz = _model.GameBoardSize;
            return Fields[x * sorHossz + y];
        }
        /// <summary>
        /// Set field in <see cref="Fields"/>.
        /// </summary>
        /// <param name="x">row count</param>
        /// <param name="y">column count</param>
        /// <param name="tile">type of tile</param>
        public void SetFieldAtCoordinates(int x, int y, MaynotTile tile)
        {
            int sorHossz = _model.GameBoardSize;
            Fields[x * sorHossz + y] = tile;
        }
        /// <summary>
        /// Set visibility of field in <see cref="Fields"/>.
        /// </summary>
        /// <param name="x">row count</param>
        /// <param name="y">column count</param>
        /// <param name="visibility">type of visibility</param>
        public void SetVisibilityAtCoordinates(int x, int y, Visibility visibility)
        {
            int sorHossz = _model.GameBoardSize;
            Fields[x * sorHossz + y].IsCurrentlyHighlighted = visibility;
        }

        private async void DebugTriggerCatastropheHappened(int x, int y)
        {
            await Task.Delay(12000); // 12 seconds delay
            //_model.catastrophe();
            CatastropheHappened(x, y); //Direktbe Viewmodel fgv.-t is hívhatunk a Debughoz
        }
        /// <summary>
        /// Changes fields in <see cref="Fields"/>.
        /// </summary>
        /// <param name="x">row count</param>
        /// <param name="y">column count</param>
        public async void CatastropheHappened(int x, int y)
        {
            int radius = 5;
            List<MaynotTile> affectedTiles = new List<MaynotTile>();

            for (int i = x - radius; i <= x + radius; i++)
            {
                for (int j = y - radius; j <= y + radius; j++)
                {
                    if (i >= 0 && j >= 0 && i < _model.GameBoardSize && j < _model.GameBoardSize)
                    {
                        MaynotTile tile = GetFieldAtCoordinates(i, j);
                        if (tile != null)
                        {
                            affectedTiles.Add(tile);
                            tile.IsFlameVisible = Visibility.Visible;
                        }
                    }
                }
            }

            OnPropertyChanged(nameof(Fields));
            currentlyAnimating = true;
            await Task.Delay(3300); // Várunk aztán eltüntetjü a Flame overlayt
            currentlyAnimating = false;
            foreach (MaynotTile tile in affectedTiles)
            {
                tile.IsFlameVisible = Visibility.Hidden;
            }

            OnPropertyChanged(nameof(Fields));
            UpdateTable();
        }
        /// <summary>
        /// Updates <see cref="ResidentalTax"/>, <see cref="IndustrialTax"/>, <see cref="ResidentalTax"/>, <see cref="Date"/> after a game has loaded.
        /// </summary>
        public void GameWasLoaded()
        {
            FullyRefreshTable();
            App.Current?.Dispatcher.Invoke((Action)delegate
            {
                OnPropertyChanged(nameof(ResidentalTax));
                OnPropertyChanged(nameof(ServiceTax));
                OnPropertyChanged(nameof(IndustrialTax));
                OnPropertyChanged(nameof(Speed));
                OnPropertyChanged(nameof(Date));
            });
        }

        #region Properties

        /// <summary>
        /// Új játék kezdése parancs lekérdezése.
        /// </summary>
        public DelegateCommand NewGameCommand { get; private set; }

        /// <summary>
        /// Játék betöltése parancs lekérdezése.
        /// </summary>
        public DelegateCommand LoadGameCommand { get; private set; }

        /// <summary>
        /// Játék mentése parancs lekérdezése.
        /// </summary>
        public DelegateCommand SaveGameCommand { get; private set; }

        /// <summary>
        /// Kilépés parancs lekérdezése.
        /// </summary>
        public DelegateCommand ExitGameCommand { get; private set; }
        /// <summary>
        /// Property getter of solwer game speed.
        /// </summary>
        public DelegateCommand SlowerGameCommand { get; private set; }
        /// <summary>
        /// Property getter of faster game speed.
        /// </summary>
        public DelegateCommand SpeedUpGameCommand { get; private set; }
        /// <summary>
        /// Property getter of pausing game speed.
        /// </summary>
        public DelegateCommand PauseGameCommand { get; private set; }
        /// <summary>
        /// Property getter of resuming game speed.
        /// </summary>
        public DelegateCommand ResumeGameCommand { get; private set; }
        /// <summary>
        /// Property getter of higher <see cref="ResidentalTax"/>.
        /// </summary>
        public DelegateCommand ResidentalTaxNumericUpCommand { get; private set; }
        /// <summary>
        /// Property getter of lower <see cref="ResidentalTax"/>.
        /// </summary>
        public DelegateCommand ResidentalTaxNumericDownCommand { get; private set; }
        /// <summary>
        /// Property getter of higher <see cref="ServiceTax"/>.
        /// </summary>
        public DelegateCommand ServiceTaxNumericUpCommand { get; private set; }
        /// <summary>
        /// Property getter of lower <see cref="ServiceTax"/>.
        /// </summary>
        public DelegateCommand ServiceTaxNumericDownCommand { get; private set; }
        /// <summary>
        /// Property getter of higher <see cref="IndustrialTax"/>.
        /// </summary>
        public DelegateCommand IndustrialTaxNumericUpCommand { get; private set; }
        /// <summary>
        /// Property getter of lower <see cref="IndustrialTax"/>.
        /// </summary>
        public DelegateCommand IndustrialTaxNumericDownCommand { get; private set; }
        /// <summary>
        /// Property getter of escaping.
        /// </summary>
        public DelegateCommand EscapeKeyPressedCommand { get; private set; }
        /// <summary>
        /// Property getter of opening help popup.
        /// </summary>
        public DelegateCommand OpenHelpPopupCommand { get; private set; }
        /// <summary>
        /// Property getter of selecting bulldozer button.
        /// </summary>
        public DelegateCommand SelectBulldozerCommand { get; private set; }
        /// <summary>
        /// Property getter of upgrading <see cref="Zone"/>.
        /// </summary>
        public DelegateCommand UpgradeInspectedZoneCommand { get; private set; }
        /// <summary>
        /// Property getter of closing transactions popup.
        /// </summary>
        public DelegateCommand CloseTransactionsPopupCommand { get; private set; }
        /// <summary>
        /// Property getter ofc opening transactions popup.
        /// </summary>
        public DelegateCommand OpenTransactionsPopupCommand { get; private set; }


        #endregion

        #region Events

        /// <summary>
        /// Új játék eseménye.
        /// </summary>
        public event EventHandler? NewGame;

        /// <summary>
        /// Játék betöltésének eseménye.
        /// </summary>
        public event EventHandler? LoadGame;

        /// <summary>
        /// Játék mentésének eseménye.
        /// </summary>
        public event EventHandler? SaveGame;

        /// <summary>
        /// Játékból való kilépés eseménye.
        /// </summary>
        public event EventHandler? ExitGame;

        #endregion

        #region Event methods

        /// <summary>
        /// Új játék indításának eseménykiváltása.
        /// </summary>
        private void OnNewGame()
        {
            NewGame?.Invoke(this, EventArgs.Empty);
        }



        /// <summary>
        /// Játék betöltése eseménykiváltása.
        /// </summary>
        private void OnLoadGame()
        {
            LoadGame?.Invoke(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játék mentése eseménykiváltása.
        /// </summary>
        private void OnSaveGame()
        {
            SaveGame?.Invoke(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játékból való kilépés eseménykiváltása.
        /// </summary>
        private void OnExitGame()
        {
            ExitGame?.Invoke(this, EventArgs.Empty);
        }

        #endregion
    }
}
