﻿using MaynotModel;
using MaynotPersistence;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Maynot.WPF.ViewModel
{
    public class Transaction: ViewModelBase
    {
        private String _message;
        /// <summary>
        /// Property of displayed message in view.
        /// </summary>
        public String Message 
        { 
            get { return _message; } 
            set
            {
                _message = value;
                OnPropertyChanged();
            }
        }
        private String _amount;
        /// <summary>
        /// Property of displayed money in view.
        /// </summary>
        public String Amount
        {
            get { return _amount; } 
            set 
            { 
                _amount = value; 
                OnPropertyChanged(); 
            } 
        }
        private String _time;
        /// <summary>
        /// Property of displayed time of transaction in view.
        /// </summary>
        public String Time
        {
            get { return _time; } 
            set 
            { 
                _time = value; 
                OnPropertyChanged(); 
            } 
        }
        private String _operator;
        /// <summary>
        /// Property of displayed type in view.
        /// </summary>
        public String Operator
        {
            get { return _operator; }
            set
            {
                _operator = value;
                OnPropertyChanged();
            }
        }
        private bool _isExpense;
        /// <summary>
        /// Property of displayed color in view.
        /// </summary>
        public bool IsExpense
        {
            get { return _isExpense; }
            set
            {
                _isExpense = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// Contruct new <see cref="Transaction"/>.
        /// </summary>
        /// <param name="type">Type of transaction</param>
        /// <param name="amount">amount of money</param>
        /// <param name="time">date of transaction</param>
        /// <param name="tileType">optional <see cref="Type"/> for building, upgrading and deleting</param>
        public Transaction(TransactionType type, float amount, DateTime time, Type tileType = null)
        {
            Amount = String.Empty;
            String tileDisplay = String.Empty;
            IsExpense = true;
            if (type == TransactionType.INDUSTRIALTAX)
            {
                tileDisplay = "Ipar zónai adók";
                IsExpense = false;
            }
            else if (type == TransactionType.RESIDENTIALTAX)
            {
                tileDisplay = "Lakó zónai adók";
                IsExpense = false;
            }
            else if (type == TransactionType.SERVICETAX)
            {
                tileDisplay = "Szolgáltatás zónai adók";
                IsExpense = false;
            }
            else if (type == TransactionType.PENSION)
            {
                tileDisplay = "Nugydíjak";
                IsExpense = true;
            }
            else if (type == TransactionType.BULLDOZER ||  type == TransactionType.UPGRADE || type == TransactionType.BUILDING)
            {
                if(tileType.Equals(typeof(MaynotPersistence.Road)))
                {
                    tileDisplay += "Út";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.Forest)))
                {
                    tileDisplay = "Erdő";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.ResidentialZone)))
                {
                    tileDisplay = "Lakó épület";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.IndustrialZone)))
                {
                    tileDisplay = "Ipari épület";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.ServiceZone)))
                {
                    tileDisplay = "Szolgáltató épület";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.PoliceStation)))
                {
                    tileDisplay = "Rendőrség";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.Stadium)))
                {
                    tileDisplay = "Stadion";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.School)))
                {
                    tileDisplay = "Iskola";
                }

                if (type == TransactionType.BUILDING)
                {
                    tileDisplay += " felépítése";
                    IsExpense = true;
                }
                else if (type == TransactionType.UPGRADE)
                {
                    tileDisplay += " fejlesztése";
                    IsExpense = true;
                }
                else if(type == TransactionType.BULLDOZER)
                {
                    tileDisplay += " eltörlése";
                    IsExpense = false;
                }
            }
            else if (type == TransactionType.MAINTENANCE)
            {
                IsExpense = true;
                if (tileType.Equals(typeof(MaynotPersistence.Road)))
                {
                    tileDisplay = "Utak";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.Forest)))
                {
                    tileDisplay = "Erdők";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.PoliceStation)))
                {
                    tileDisplay = "Rendőrségek";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.Stadium)))
                {
                    tileDisplay = "Stadionok";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.School)))
                {
                    tileDisplay = "Iskolák";
                }
                else if (tileType.Equals(typeof(MaynotPersistence.School)))
                {
                    tileDisplay = "Egyetemek";
                }
                tileDisplay += " fenntartása";
            }
            Operator = IsExpense ? "-" : "+";
            Message = tileDisplay;
            Amount += amount.ToString($"F{3}");
            Time = time.ToString("yyyy MMMM dd");
        }
    }
}
