﻿using System;
using System.Globalization;
using System.IO;
using System.Threading.Tasks;

namespace MaynotPersistence
{
	public class FilePersistence : IPersistence
	{
		public async Task SaveAsync(string path, MaynotGameState state)
		{
			try
			{
				using StreamWriter writer = new(path);
				await writer.WriteAsync(state.time.ToString("yyyy-MM-dd") + " " +
										state.size.ToString() + " " +
										state.startX.ToString() + " " +
										state.startY.ToString() + " " +
										state.yearTracker.ToString() + " " +
										state.weakTracker.ToString() + " " +
										state.guaranteedPopulation.ToString() + " " +
										state.income.ToString() + " " +
										state.expense.ToString() + " " +
										state.money.ToString() + " " +
										state.ResidentalTax.ToString() + " " +
										state.ServiceTax.ToString() + " " +
										state.IndustrialTax.ToString() + " " +
										state.citizens.Count.ToString() + " " +
										state.MaintenanceFees.Count.ToString() + " " +
										// folytatás, ha kell
										"\n");
				
				foreach (Person p in state.citizens)
				{
					await writer.WriteAsync(p.Satisfaction.ToString() + " " +
												p.Age.ToString() + " " +
												p.Residency.Item1.ToString() + " " +
												p.Residency.Item2.ToString() + " " +
												p.WorkPlace.Item1.ToString() + " " +
												p.WorkPlace.Item2.ToString() + " " +
												p.Education.ToString() + " ");

					foreach (float tax in p.Taxes)
					{
						await writer.WriteAsync(tax.ToString() + " ");
					}
					await writer.WriteAsync("\n");
				}

				foreach(Type tileType in state.MaintenanceFees.Keys)
				{
                    await writer.WriteAsync(tileType.Name + " " + state.MaintenanceFees[tileType].ToString() + " ");
                    await writer.WriteAsync("\n");
                }
				for (int i = 0; i < state.size; i++)
				{
					for (int j = 0; j < state.size; j++)
					{
						if (state.gameBoard[i, j] is ResidentialZone resZone)
						{
							await writer.WriteAsync("ResidentialZone " + i + " " + j + " " +
													resZone.Level.ToString() + " " + resZone.Capacity.ToString() + " "
													+ "\n");
						}
						if (state.gameBoard[i, j] is ServiceZone serZone)
						{
							await writer.WriteAsync("ServiceZone " + i + " " + j + " " +
													serZone.Level.ToString() + " " + serZone.Capacity.ToString() + " "
													+ "\n");
						}
						if (state.gameBoard[i, j] is IndustrialZone indZone)
						{
							await writer.WriteAsync("IndustrialZone " + i + " " + j + " " +
													indZone.Level.ToString() + " " + indZone.Capacity.ToString() + " "
													+ "\n");
						}
						if (state.gameBoard[i, j] is Forest forZone)
						{
							await writer.WriteAsync("Forest " + i + " " + j + " " +
													forZone.PlantingDate.ToString("yyyy-MM-dd") + " " +
													forZone.Age.ToString() + " " +
													forZone.Generated.ToString() + "\n");
						}
						if (state.gameBoard[i, j] is PoliceStation polZone)
						{
							await writer.WriteAsync("Police " + i + " " + j + "\n");
						}
						if (state.gameBoard[i, j] is Stadium stadZone)
						{
							await writer.WriteAsync("Stadium " + i + " " + j + " " + stadZone.Id + "\n");
						}
						if (state.gameBoard[i, j] is Road roadZone)
						{
							await writer.WriteAsync("Road " + i + " " + j + "\n");
						}
						if (state.gameBoard[i, j] is School schoolZone)
						{
							await writer.WriteAsync("School " + i + " " + j + " " + schoolZone.Id + "\n");
						}
						if (state.gameBoard[i, j] is University univerZone)
						{
							await writer.WriteAsync("University " + i + " " + j + " " + univerZone.Id + "\n");
						}
					}
				}
			}
			catch
			{
				throw new DataException();
			}
		}
		public async Task<MaynotGameState> LoadAsync(string path)
		{
			try
			{
				using StreamReader reader = new(path);
				MaynotGameState state = new(30);

				string? v = await reader.ReadLineAsync();
				if (v != null)
				{
					string[] header = v.Split(' ');
					state.gameSpeed = 0;
					state.prevGameSpeed = 1;
					state.time = DateTime.ParseExact(header[0], "yyyy-MM-dd", CultureInfo.InvariantCulture);
					state.size = int.Parse(header[1]);
					state.startX = int.Parse(header[2]);
					state.startY = int.Parse(header[3]);
					state.yearTracker = int.Parse(header[4]);
					state.weakTracker = int.Parse(header[5]);
					state.guaranteedPopulation = bool.Parse(header[6]);
					state.income = int.Parse(header[7]);
					state.expense = int.Parse(header[8]);
					state.money = int.Parse(header[9]);
					state.ResidentalTax = int.Parse(header[10]);
					state.ServiceTax = int.Parse(header[11]);
					state.IndustrialTax = int.Parse(header[12]);
					int citizensCount = int.Parse(header[13]);
					int feeCount = int.Parse(header[14]);

                    for (int i = 0; i < citizensCount; i++)
					{
						string? c = await reader.ReadLineAsync();
						string[] citizenData;

						if (c != null)
						{
							citizenData = c.Split(" ");

							int satisfaction = int.Parse(citizenData[0]);
							int age = int.Parse(citizenData[1]);
							int resX = int.Parse(citizenData[2]);
							int resY = int.Parse(citizenData[3]);
							int workX = int.Parse(citizenData[4]);
							int workY = int.Parse(citizenData[5]);
							Level education = (Level)Enum.Parse(typeof(Level), citizenData[6]);

							Person citizen = new(satisfaction, age, (resX, resY), (workX, workY), education);
							Array.Copy(Array.ConvertAll(new ArraySegment<string>(citizenData, 7, 20).ToArray(), float.Parse), 0, citizen.Taxes, 0, 20);

							state.citizens.Add(citizen);
						}
					}
                    for (int i = 0; i < feeCount; i++)
                    {
                        string? line = await reader.ReadLineAsync();
                        string[] feeData;

                        if (line != null)
                        {
                            feeData = line.Split(" ");
                            if	(feeData[0] == "Road")
							{
								state.MaintenanceFees.TryAdd(typeof(Road), float.Parse(feeData[1]));
							} 
							else if (feeData[0] == "Forest")
                            {
                                state.MaintenanceFees.TryAdd(typeof(Forest), float.Parse(feeData[1]));
                            }
                            else if (feeData[0] == "School")
                            {
                                state.MaintenanceFees.TryAdd(typeof(School), float.Parse(feeData[1]));
                            }
                            else if (feeData[0] == "University")
                            {
                                state.MaintenanceFees.TryAdd(typeof(University), float.Parse(feeData[1]));
                            }
                            else if (feeData[0] == "PoliceStation")
                            {
                                state.MaintenanceFees.TryAdd(typeof(PoliceStation), float.Parse(feeData[1]));
                            }
                            else if (feeData[0] == "Stadium")
                            {
                                state.MaintenanceFees.TryAdd(typeof(Stadium), float.Parse(feeData[1]));
                            }
                        }
                    }
                    while (!reader.EndOfStream)
					{
						string? line = await reader.ReadLineAsync();
						string[] tileData;
						if (line != null)
						{
							tileData = line.Split(" ");
							string type = tileData[0];
							if (type == "Forest")
							{
								int x = int.Parse(tileData[1]);
								int y = int.Parse(tileData[2]);
								DateTime dateTime = DateTime.ParseExact(tileData[3], "yyyy-MM-dd", CultureInfo.InvariantCulture);
								bool generated = tileData[5] == "True";

								Forest f = new(dateTime, generated);
								state.gameBoard[x, y] = f;
							}
							if (type == "Road")
							{
								int x = int.Parse(tileData[1]);
								int y = int.Parse(tileData[2]);

								Road r = new();
								state.gameBoard[x, y] = r;
							}
							if (type == "ResidentialZone")
							{
								int x = int.Parse(tileData[1]);
								int y = int.Parse(tileData[2]);
								ZoneLevel level = (ZoneLevel)Enum.Parse(typeof(ZoneLevel), tileData[3]);
								int capacity = int.Parse(tileData[4]);

								ResidentialZone r = new(x, y, level, capacity);
								state.residentalZones.Add((x, y));
								state.gameBoard[x, y] = r;
							}
							if (type == "ServiceZone")
							{
								int x = int.Parse(tileData[1]);
								int y = int.Parse(tileData[2]);
								ZoneLevel level = (ZoneLevel)Enum.Parse(typeof(ZoneLevel), tileData[3]);
								int capacity = int.Parse(tileData[4]);

								ServiceZone r = new(x, y, level, capacity);
								state.serviceZones.Add((x, y));
								state.gameBoard[x, y] = r;
							}
							if (type == "IndustrialZone")
							{
								int x = int.Parse(tileData[1]);
								int y = int.Parse(tileData[2]);
								ZoneLevel level = (ZoneLevel)Enum.Parse(typeof(ZoneLevel), tileData[3]);
								int capacity = int.Parse(tileData[4]);

								IndustrialZone r = new(x, y, level, capacity);
								state.industrialZones.Add((x, y));
								state.gameBoard[x, y] = r;
							}
							if (type == "Police")
							{
								PoliceStation p = new();
								state.gameBoard[int.Parse(tileData[1]), int.Parse(tileData[2])] = p;
							}
							if (type == "University")
							{
								University p = new(int.Parse(tileData[3]));
								state.gameBoard[int.Parse(tileData[1]), int.Parse(tileData[2])] = p;
							}
							if (type == "School")
							{
								School p = new(int.Parse(tileData[3]));
								state.gameBoard[int.Parse(tileData[1]), int.Parse(tileData[2])] = p;
							}
							if (tileData[0] == "Stadium")
							{
								Stadium p = new(int.Parse(tileData[3]));
								state.gameBoard[int.Parse(tileData[1]), int.Parse(tileData[2])] = p;
							}
						}
					}
				}

				return state;
			}
			catch
			{
				throw new DataException();
			}
		}
	}
}
