﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace MaynotPersistence
{
    public enum ZoneLevel
    {
        SMALL = 0,
        MEDIUM = 1,
        LARGE = 2,
    }
	public interface Zone : Tile
	{
		Tuple<int, int> Coordinates { get; set; }
		ZoneLevel Level { get; set; }
		int Capacity { get; set; }
		int BuildCost { get; }
		int UpgradeCost { get; }
		List<Person> GetPeoples(List<Person> citizens);
		void Upgrade();
	}

	public class ResidentialZone: Zone
    {
		public Tuple<int, int> Coordinates { get; set; }
		public ZoneLevel Level { get; set; }
        public int Capacity { get; set; }
		public int BuildCost { get; } = 750;
		public int UpgradeCost { get; } = 650;

		public ResidentialZone(int x, int y, ZoneLevel level, int capacity)
        {
            Coordinates = new Tuple<int, int>(x, y);
			Level = level;
			Capacity = capacity;
        }

		public List<Person> GetPeoples(List<Person> citizens)
		{
			List<Person> selectedCitizens = citizens.Where(p => p.Residency.Item1 == Coordinates.Item1 && p.Residency.Item2 == Coordinates.Item2).ToList();
			return selectedCitizens;
		}
		public void Upgrade()
        {
            if (Level == ZoneLevel.SMALL)
            {
                Level = ZoneLevel.MEDIUM;
                Capacity *= 2;
            }
            else if (Level == ZoneLevel.MEDIUM)
            {
                Level = ZoneLevel.LARGE;
                Capacity *= 2;
            }
        }
    }
    public class IndustrialZone : Zone
    {
		public Tuple<int, int> Coordinates { get; set; }
		public ZoneLevel Level { get; set; }
		public int Capacity { get; set; }
		public int BuildCost { get; } = 750;
		public int UpgradeCost { get; } = 650;

		public IndustrialZone(int x, int y, ZoneLevel level, int capacity)
		{
			Coordinates = new Tuple<int, int>(x, y);
			Level = level;
			Capacity = capacity;
		}

		public List<Person> GetPeoples(List<Person> citizens)
		{
			List<Person> selectedCitizens = citizens.Where(p => p.WorkPlace.Item1 == Coordinates.Item1 && p.WorkPlace.Item2 == Coordinates.Item2).ToList();
			return selectedCitizens;
		}
		public void Upgrade()
		{
			if (Level == ZoneLevel.SMALL)
			{
				Level = ZoneLevel.MEDIUM;
				Capacity *= 2;
			}
			else if (Level == ZoneLevel.MEDIUM)
			{
				Level = ZoneLevel.LARGE;
				Capacity *= 2;
			}
		}
	}
    public class ServiceZone : Zone
    {
		public Tuple<int, int> Coordinates { get; set; }
		public ZoneLevel Level { get; set; }
		public int Capacity { get; set; }
		public int BuildCost { get; } = 750;
		public int UpgradeCost { get; } = 650;

		public ServiceZone(int x, int y, ZoneLevel level, int capacity)
		{
			Coordinates = new Tuple<int, int>(x, y);
			Level = level;
			Capacity = capacity;
		}

		public List<Person> GetPeoples(List<Person> citizens)
		{
			List<Person> selectedCitizens = citizens.Where(p => p.WorkPlace.Item1 == Coordinates.Item1 && p.WorkPlace.Item2 == Coordinates.Item2).ToList();
			return selectedCitizens;
		}
		public void Upgrade()
		{
			if (Level == ZoneLevel.SMALL)
			{
				Level = ZoneLevel.MEDIUM;
				Capacity *= 2;
			}
			else if (Level == ZoneLevel.MEDIUM)
			{
				Level = ZoneLevel.LARGE;
				Capacity *= 2;
			}
		}
	}
}
