using MaynotModel;
using MaynotPersistence;
using Moq;
using System.Data;
using System.Text.Json;

namespace MaynotTest
{
    [TestFixture]
    public class Tests
    {
        private Mock<IPersistence> _mock = null!;
        private MaynotGameModel _model = null!;
        [SetUp]
        public void Setup()
        {
            _mock = new Mock<IPersistence>();
            _model = new MaynotGameModel(_mock.Object);
        }
        [Test]
        public void ConstructorTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                    Assert.True(_model.GameBoard[i, j] is Tile);

        }
        [Test]
        public void NewGamePopulationIsZeroTest()
        {
            _model.NewGame();
            Assert.That(_model.Population, Is.EqualTo(0));
            Assert.Pass();
        }
        public void NewGameHasStartingRoadTest()
        {
            _model.NewGame();
            bool hasStartingRoad = false;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        hasStartingRoad = true;
                    }
                }
            Assert.IsTrue(hasStartingRoad);
        }
        [Test]
        public void PlaceRoadTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        Assert.That(_model.placeRoad(i, j), Is.EqualTo(true));
                        budget -= Road.buildCost;
                    }
                }
            Assert.That(_model.Transactions[0].Type, Is.EqualTo(TransactionType.BUILDING));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(Road.buildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(Road)));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void PlaceForestTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        Assert.That(_model.placeForest(i, j), Is.EqualTo(true));
                        budget -= Forest.BuildCost;
                    }
                    else if (_model.GameBoard[i, j] is Forest)
                    {
                        Assert.That(((Forest)_model.GameBoard[i, j]).Generated, Is.EqualTo(true));
                        Assert.That(((Forest)_model.GameBoard[i, j]).PlantingDate, Is.EqualTo(_model.Time));
                        Assert.That(((Forest)_model.GameBoard[i, j]).Age, Is.EqualTo(0));
                    }
                }
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(Forest)));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(Forest.BuildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void PlaceIndustrialZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        _model.placeIndustrialZone(i, j);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is not Forest && _model.GameBoard[i, j] is not Road)
                    {
                        Assert.IsTrue(_model.GameBoard[i, j] is IndustrialZone);
                    }
                }
        }
        [Test]
        public void PlaceResidentialZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        _model.placeResidentialZone(i, j);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is not Forest && _model.GameBoard[i, j] is not Road)
                    {
                        Assert.IsTrue(_model.GameBoard[i, j] is ResidentialZone);
                    }
                }
        }
        [Test]
        public void PlaceServiceZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        _model.placeServiceZone(i, j);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is not Forest && _model.GameBoard[i, j] is not Road)
                    {
                        Assert.IsTrue(_model.GameBoard[i, j] is ServiceZone);
                    }
                }
        }
        [Test]
        public void PlacePoliceStationTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        Assert.That(_model.placePoliceStation(i, j), Is.EqualTo(true));
                        budget -= PoliceStation.buildCost;
                    }
                }
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(PoliceStation)));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(PoliceStation.buildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void PlaceSchoolTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty && i + 1 < _model.GameBoardSize && j + 1 < _model.GameBoardSize && _model.GameBoard[i + 1, j] is Empty)
                    {
                        Assert.That(_model.placeSchool(i, j), Is.EqualTo(true));
                        Assert.That(((School)_model.GameBoard[i, j]).People.Count, Is.EqualTo(0));
                        budget -= School.buildCost;
                    }
                }
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(School)));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(School.buildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void PlaceUniversityTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty && i + 1 < _model.GameBoardSize && j + 1 < _model.GameBoardSize && _model.GameBoard[i + 1, j] is Empty && _model.GameBoard[i, j + 1] is Empty && _model.GameBoard[i + 1, j + 1] is Empty)
                    {
                        Assert.That(_model.placeUni(i, j), Is.EqualTo(true));
                        Assert.That(((University)_model.GameBoard[i, j]).People.Count, Is.EqualTo(0));
                        budget -= University.buildCost;
                    }
                }
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(University)));
            Assert.That(_model.Transactions[0].Type, Is.EqualTo(TransactionType.BUILDING));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(University.buildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void PlaceStadiumTest()
        {
            _model.NewGame();
            float budget = _model.Money;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty && i + 1 < _model.GameBoardSize && j + 1 < _model.GameBoardSize && _model.GameBoard[i + 1, j] is Empty && _model.GameBoard[i, j + 1] is Empty && _model.GameBoard[i + 1, j + 1] is Empty)
                    {
                        Assert.That(_model.placeStadium(i, j), Is.EqualTo(true));
                        budget -= Stadium.buildCost;
                    }
                }
            Assert.That(_model.Transactions[0].TileType, Is.EqualTo(typeof(Stadium)));
            Assert.That(_model.Transactions[0].TransactionAmount, Is.EqualTo(Stadium.buildCost));
            Assert.That(_model.Transactions[0].TransactionDate, Is.EqualTo(_model.Time));
            Assert.That(budget, Is.EqualTo(_model.Money));
        }
        [Test]
        public void DestroyRoadTest()
        {
            _model.NewGame();
            int x = 0;
            int y = 0;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        x = i;
                        y = j;
                    }
                }
            Assert.That(_model.DestroyRoad(x, y), Is.EqualTo(false));

            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        _model.placeRoad(i, j);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        _model.DestroyRoad(i, j);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (i != _model.GetStartRoadX() || j != _model.GetStartRoadY())
                    {
                        Assert.IsFalse(_model.GameBoard[i, j] is Road);
                    }
                    else
                    {
                        Assert.IsTrue(_model.GameBoard[i, j] is Road);
                    }
                }

            if (_model.GameBoard[x - 1, y] is Empty && _model.GameBoard[x - 2, y] is Empty)
            {
                _model.placeRoad(x - 1, y);
                _model.placePoliceStation(x - 2, y);
                Assert.That(_model.DestroyRoad(x - 1, y), Is.EqualTo(false));
            }
            if (_model.GameBoard[x + 1, y] is Empty && _model.GameBoard[x + 2, y] is Empty)
            {
                _model.placeRoad(x + 1, y);
                _model.placeUni(x + 2, y);
                Assert.That(_model.DestroyRoad(x + 1, y), Is.EqualTo(false));
            }
            if (_model.GameBoard[x, y + 1] is Empty && _model.GameBoard[x, y + 2] is Empty)
            {
                _model.placeRoad(x, y + 1);
                _model.placeSchool(x, y + 2);
                Assert.That(_model.DestroyRoad(x, y + 1), Is.EqualTo(false));
            }
        }
        [Test]
        public void SpeedUpTimeTest()
        {
            _model.NewGame();
            Assert.That(_model.Speed, Is.EqualTo(0));
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(1));
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(2));
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(3));

        }
        [Test]
        public void SlowerTimeTest()
        {
            _model.NewGame();
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(1));
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(2));
            _model.slowTime();
            Assert.That(_model.Speed, Is.EqualTo(1));
            _model.slowTime();
            Assert.That(_model.Speed, Is.EqualTo(0));
        }
        [Test]
        public void StopAndResumeTimeTest()
        {
            _model.NewGame();
            _model.speedUpTime();
            Assert.That(_model.Speed, Is.EqualTo(1));
            _model.stopTime();
            Assert.That(_model.Speed, Is.EqualTo(0));
            _model.resumeTime();
            Assert.That(_model.Speed, Is.EqualTo(1));
        }
        [Test]
        public void UpdateServiceZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        var startingBudget = _model.Money;
                        _model.placeServiceZone(i, j);
                        _model.InspectZone(i, j);
                        ServiceZone zone = (ServiceZone)_model.GameBoard[i, j];
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.SMALL));
                        Assert.That(zone.BuildCost, Is.EqualTo(750));
                        Assert.That(zone.UpgradeCost, Is.EqualTo(650));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.MEDIUM));
                        Assert.That(_model.Transactions[0].Type, Is.EqualTo(TransactionType.UPGRADE));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.LARGE));
                        Assert.That(_model.Transactions[1].Type, Is.EqualTo(TransactionType.UPGRADE));
                        var budget = _model.Money + 2.0 * zone.UpgradeCost;
                        Assert.That(budget, Is.EqualTo(startingBudget));
                        return;
                    }
                }
        }
        [Test]
        public void UpdateResidentialZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        var startingBudget = _model.Money;
                        _model.placeResidentialZone(i, j);
                        _model.InspectZone(i, j);
                        ResidentialZone zone = (ResidentialZone)_model.GameBoard[i, j];
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.SMALL));
                        Assert.That(zone.BuildCost, Is.EqualTo(750));
                        Assert.That(zone.UpgradeCost, Is.EqualTo(650));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.MEDIUM));
                        Assert.That(_model.Transactions[0].Type, Is.EqualTo(TransactionType.UPGRADE));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.LARGE));
                        Assert.That(_model.Transactions[1].Type, Is.EqualTo(TransactionType.UPGRADE));
                        var budget = _model.Money + 2.0 * zone.UpgradeCost;
                        Assert.That(budget, Is.EqualTo(startingBudget));
                        return;
                    }
                }
        }
        [Test]
        public void UpdateIndustrialZoneTest()
        {
            _model.NewGame();
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Empty)
                    {
                        var startingBudget = _model.Money;
                        _model.placeIndustrialZone(i, j);
                        _model.InspectZone(i, j);
                        IndustrialZone zone = (IndustrialZone)_model.GameBoard[i, j];
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.SMALL));
                        Assert.That(zone.BuildCost, Is.EqualTo(750));
                        Assert.That(zone.UpgradeCost, Is.EqualTo(650));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.MEDIUM));
                        Assert.That(_model.Transactions[0].Type, Is.EqualTo(TransactionType.UPGRADE));
                        _model.UpgradeInspectedZone();
                        Assert.That(zone.Level, Is.EqualTo(ZoneLevel.LARGE));
                        Assert.That(_model.Transactions[1].Type, Is.EqualTo(TransactionType.UPGRADE));
                        var budget = _model.Money + 2.0 * zone.UpgradeCost;
                        Assert.That(budget, Is.EqualTo(startingBudget));
                        return;
                    }
                }
        }
        [Test]
        public void InspectZoneTest()
        {
            _model.NewGame();
            _model.InspectZone(1, 2);
            Assert.That(_model.InspectedTile, Is.EqualTo(_model.GameBoard[1, 2]));
        }
        [Test]
        public void SetTaxesTest()
        {
            _model.NewGame();
            _model.setIndustrialTax(1);
            Assert.That(_model.IndustrialTax, Is.EqualTo(1));
            _model.setIndustrialTax(42);
            Assert.That(_model.IndustrialTax, Is.EqualTo(42));
            _model.setResidentalTax(7);
            Assert.That(_model.ResidentalTax, Is.EqualTo(7));
            _model.setResidentalTax(45);
            Assert.That(_model.ResidentalTax, Is.EqualTo(45));
            _model.setServiceTax(70);
            Assert.That(_model.ServiceTax, Is.EqualTo(70));
            _model.setServiceTax(12);
            Assert.That(_model.ServiceTax, Is.EqualTo(12));
        }
        [Test]
        public void MovingInTest()
        {
            _model.NewGame();
            int x = 0;
            int y = 0;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Forest)
                    {
                        _model.DestroyTile(i, j, false);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        x = i;
                        y = j;
                    }
                }
            if (_model.GameBoard[x - 1, y] is Empty && _model.GameBoard[x + 1, y] is Empty)
            {
                _model.placeResidentialZone(x - 1, y);
                _model.placeServiceZone(x + 1, y);
            }
            Assert.That(((Zone)_model.GameBoard[x - 1, y]).GetPeoples(_model.Citizens).Count, Is.EqualTo(0));
            Assert.That(((Zone)_model.GameBoard[x + 1, y]).GetPeoples(_model.Citizens).Count, Is.EqualTo(0));

            int daysToAdd = ((int)DayOfWeek.Sunday - (int)_model.Time.DayOfWeek + 7) % 7;
            _model.Time = _model.Time.AddDays(daysToAdd);
            _model.speedUpTime();
            while (_model.Time.Day < 8)
            {
                continue;
            }
            _model.stopTime();
            Assert.That(_model.Population, Is.Not.EqualTo(0));
            Assert.That(_model.AverageSatisfaction, Is.Not.EqualTo(0));
            Assert.That(_model.Citizens.Count, Is.Not.EqualTo(0));
            Assert.That(((Zone)_model.GameBoard[x - 1, y]).GetPeoples(_model.Citizens).Count, Is.Not.EqualTo(0));
            Assert.That(((Zone)_model.GameBoard[x + 1, y]).GetPeoples(_model.Citizens).Count, Is.Not.EqualTo(0));
        }
        [Test]
        public void MockCatastropheTest()
        {
            _model.NewGame();
            int x = 0;
            int y = 0;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Forest)
                    {
                        _model.DestroyTile(i, j, false);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        x = i;
                        y = j;
                    }
                }
            if (_model.GameBoard[x - 1, y] is Empty && _model.GameBoard[x + 1, y] is Empty)
            {
                _model.placeResidentialZone(x - 1, y);
                _model.placeServiceZone(x + 1, y);
            }
            Assert.That(_model.GameBoard[x - 1, y].GetType(), Is.EqualTo(typeof(ResidentialZone)));
            Assert.That(_model.GameBoard[x + 1, y].GetType(), Is.EqualTo(typeof(ServiceZone)));
            _model.DestroyTile(x - 1, y, true);
            _model.DestroyTile(x + 1, y, true);
            Assert.That(_model.GameBoard[x - 1, y].GetType(), Is.EqualTo(typeof(Empty)));
            Assert.That(_model.GameBoard[x + 1, y].GetType(), Is.EqualTo(typeof(Empty)));
        }
        [Test]
        public void PensionTest()
        {
            _model.NewGame();
            int x = 0;
            int y = 0;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Forest)
                    {
                        _model.DestroyTile(i, j, false);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        x = i;
                        y = j;
                    }
                }
            if (_model.GameBoard[x - 1, y] is Empty && _model.GameBoard[x + 1, y] is Empty)
            {
                _model.placeResidentialZone(x - 1, y);
                _model.placeServiceZone(x + 1, y);
            }
            int daysToAdd = ((int)DayOfWeek.Sunday - (int)_model.Time.DayOfWeek + 7) % 7;
            _model.Time = _model.Time.AddDays(daysToAdd);
            _model.speedUpTime();
            while (_model.Time.Day < 8)
            {
                continue;
            }
            _model.stopTime();
            _model.Citizens[0].Age = 61;
            for (int i = 0; i < _model.Citizens[0].Taxes.Length-1; i++)
            {
                _model.Citizens[0].Taxes[i] = 100;
            }
            DateTime nextDecember31 = new DateTime(_model.Time.Year, 12, 31);
            daysToAdd = (nextDecember31 - _model.Time).Days;
            _model.Time = _model.Time.AddDays(daysToAdd);
            _model.managingPension();
            Assert.That(_model.Transactions.Any(transaction => transaction.Type == TransactionType.PENSION), Is.EqualTo(true));
        }
        [Test]
        public void CitizenTest()
        {
            _model.NewGame();
            int x = 0;
            int y = 0;
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Forest)
                    {
                        _model.DestroyTile(i, j, false);
                    }
                }
            for (int i = 0; i < _model.GameBoardSize; i++)
                for (int j = 0; j < _model.GameBoardSize; j++)
                {
                    if (_model.GameBoard[i, j] is Road)
                    {
                        x = i;
                        y = j;
                    }
                }
            if (_model.GameBoard[x - 1, y] is Empty && _model.GameBoard[x + 1, y] is Empty)
            {
                _model.placeResidentialZone(x - 1, y);
                _model.placeServiceZone(x + 1, y);
            }
            Assert.That(((Zone)_model.GameBoard[x - 1, y]).GetPeoples(_model.Citizens).Count, Is.EqualTo(0));
            Assert.That(((Zone)_model.GameBoard[x + 1, y]).GetPeoples(_model.Citizens).Count, Is.EqualTo(0));

            int daysToAdd = ((int)DayOfWeek.Sunday - (int)_model.Time.DayOfWeek + 7) % 7;
            _model.Time = _model.Time.AddDays(daysToAdd);
            _model.speedUpTime();
            while (_model.Time.Day < 8)
            {
                continue;
            }
            _model.stopTime();
            foreach(Person p in _model.Citizens)
            {
                Assert.That(p.Residency, Is.EqualTo((x-1,y)));
                Assert.That(p.WorkPlace, Is.EqualTo((x+1,y)));
                Assert.That(p.Satisfaction, Is.Not.EqualTo(0));
                Assert.That(p.Taxes.Length, Is.EqualTo(20));
            }
        }
    }
}
